@@ -28,7 +28,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <sys/mman.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <dirent.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -38,6 +38,7 @@
 #include <zlib.h>
 
 #include "cramfs.h"
+#include "cramfs_common.h"
 #include "md5.h"
 #include "nls.h"
 
@@ -55,6 +56,7 @@ static int verbose = 0;
 static unsigned int blksize; /* settable via -b option */
 static long total_blocks = 0, total_nodes = 1; /* pre-count the root node */
 static int image_length = 0;
+static int cramfs_is_big_endian = 0; /* target is big endian */
 
 /*
  * If opt_holes is set, then mkcramfs can create explicit holes in the
@@ -121,7 +123,7 @@ usage(int status) {
 	FILE *stream = status ? stderr : stdout;
 
 	fprintf(stream,
-		_("usage: %s [-h] [-v] [-b blksize] [-e edition] [-i file] "
+		_("usage: %s [-h] [-v] [-b blksize] [-e edition] [-N endian] [-i file] "
 		  "[-n name] dirname outfile\n"
 		  " -h         print this help\n"
 		  " -v         be verbose\n"
@@ -129,6 +131,7 @@ usage(int status) {
 		    "(non-zero exit status)\n"
 		  " -b blksize use this blocksize, must equal page size\n"
 		  " -e edition set edition number (part of fsid)\n"
+		  " -N endian  set cramfs endianness (big|little|host), default host\n"
 		  " -i file    insert a file image into the filesystem "
 		    "(requires >= 2.4.0)\n"
 		  " -n name    set name of cramfs filesystem\n"
@@ -244,7 +247,7 @@ identical_file(struct entry *e1, struct entry *e2){
  */
 #define MAX_INPUT_NAMELEN 255
 
-static int find_identical_file(struct entry *orig, struct entry *new)
+static int find_identical_file(struct entry *orig, struct entry *new, loff_t *fslen_ub)
 {
         if (orig == new)
 		return 1;
@@ -261,19 +264,20 @@ static int find_identical_file(struct entry *orig, struct entry *new)
 		    !memcmp(orig->md5sum, new->md5sum, 16) &&
 		    identical_file(orig, new)) {
 			new->same = orig;
+			*fslen_ub -= new->size;
 			return 1;
 		}
         }
-        return find_identical_file(orig->child, new) ||
-                   find_identical_file(orig->next, new);
+        return find_identical_file(orig->child, new, fslen_ub) ||
+                   find_identical_file(orig->next, new, fslen_ub);
 }
 
-static void eliminate_doubles(struct entry *root, struct entry *orig) {
+static void eliminate_doubles(struct entry *root, struct entry *orig, loff_t *fslen_ub) {
         if (orig) {
                 if (orig->size && orig->path)
-			find_identical_file(root,orig);
-                eliminate_doubles(root,orig->child);
-                eliminate_doubles(root,orig->next);
+			find_identical_file(root,orig, fslen_ub);
+                eliminate_doubles(root,orig->child, fslen_ub);
+                eliminate_doubles(root,orig->next, fslen_ub);
         }
 }
 
@@ -454,17 +458,22 @@ static unsigned int write_superblock(struct entry *root, char *base, int size)
 	super->root.size = root->size;
 	super->root.offset = offset >> 2;
 
+	super_toggle_endianness(cramfs_is_big_endian, super);
+	inode_from_host(cramfs_is_big_endian, &super->root, &super->root);
+
 	return offset;
 }
 
 static void set_data_offset(struct entry *entry, char *base, unsigned long offset)
 {
 	struct cramfs_inode *inode = (struct cramfs_inode *) (base + entry->dir_offset);
+	inode_to_host(cramfs_is_big_endian, inode, inode);
 	if (offset >= (1 << (2 + CRAMFS_OFFSET_WIDTH))) {
 		fprintf(stderr, _("filesystem too big.  Exiting.\n"));
 		exit(8);
 	}
 	inode->offset = (offset >> 2);
+	inode_from_host(cramfs_is_big_endian, inode, inode);
 }
 
 
@@ -523,6 +532,7 @@ static unsigned int write_directory_structure(struct entry *entry, char *base, u
 				entry_stack[stack_entries] = entry;
 				stack_entries++;
 			}
+			inode_from_host(cramfs_is_big_endian, inode, inode);
 			entry = entry->next;
 		}
 
@@ -630,7 +640,7 @@ do_compress(char *base, unsigned int offset, unsigned char const *name,
 			exit(8);
 		}
 
-		*(u32 *) (base + offset) = curr;
+		*(u32 *) (base + offset) = u32_toggle_endianness(cramfs_is_big_endian, curr);
 		offset += 4;
 	} while (size);
 
@@ -734,6 +744,7 @@ int main(int argc, char **argv)
 	char const *dirname, *outfile;
 	u32 crc = crc32(0L, Z_NULL, 0);
 	int c;
+	cramfs_is_big_endian = HOST_IS_BIG_ENDIAN; /* default is to use host order */
 
 	blksize = getpagesize();
 	total_blocks = 0;
@@ -750,7 +761,7 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 
 	/* command line options */
-	while ((c = getopt(argc, argv, "hb:Ee:i:n:psVvz")) != EOF) {
+	while ((c = getopt(argc, argv, "hb:Ee:i:n:N:psVvz")) != EOF) {
 		switch (c) {
 		case 'h':
 			usage(0);
@@ -764,6 +775,20 @@ int main(int argc, char **argv)
 			break;
 		case 'e':
 			opt_edition = atoi(optarg);
+			break;
+		case 'N':
+			if (strcmp(optarg, "big") == 0)  {
+				cramfs_is_big_endian = 1;
+			}
+			else if (strcmp(optarg, "little") == 0) {
+				cramfs_is_big_endian = 0;
+			}
+			else if (strcmp(optarg, "host") == 0);	/* default */
+			else 	{
+				perror("invalid endianness given. Must be 'big', 'little', or 'host'");
+				exit(16);
+			}
+
 			break;
 		case 'i':
 			opt_image = optarg;
@@ -835,7 +860,7 @@ int main(int argc, char **argv)
 	}
 
         /* find duplicate files */
-        eliminate_doubles(root_entry,root_entry);
+        eliminate_doubles(root_entry,root_entry, &fslen_ub);
 
 	/* TODO: Why do we use a private/anonymous mapping here
            followed by a write below, instead of just a shared mapping
@@ -891,7 +916,7 @@ int main(int argc, char **argv)
 
 	/* Put the checksum in. */
 	crc = crc32(crc, (unsigned char *) (rom_image+opt_pad), (offset-opt_pad));
-	((struct cramfs_super *) (rom_image+opt_pad))->fsid.crc = crc;
+	((struct cramfs_super *) (rom_image+opt_pad))->fsid.crc = u32_toggle_endianness(cramfs_is_big_endian, crc);
 	if (verbose)
 		printf(_("CRC: %x\n"), crc);
 
